/***************
adevs, a library supporting discrete event simulation
Copyright (C) 1999, 2000, 2001 James Nutaro

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Bugs, comments, and questions can be sent to nutaro@ece.arizona.edu
***************/
/**
Simulator for devsn models.  This class is used internally.
*/

class DEVScoordinator: public simulator {

public:		DEVScoordinator():simulator(simulator::COORD) {
			model = NULL;
			}
		virtual void computeNextOutput (stime_t t);
		virtual void computeIO (stime_t t);
		virtual void computeTransition (stime_t t);
/// Rebuild the schedule based on the current state of the network model.
		virtual void reset (stime_t t0 = 0.0);
		virtual void input (port_t inport, object* value);
		virtual void endOfCycle (indexed& garbage);
		virtual void setModel (devsn* model);
		virtual void clear_schedule ();
		virtual ~DEVScoordinator ();

protected:
/// Protected constructor for derived types to set SimType
		DEVScoordinator(SimType type):simulator(type) {
			model = NULL;
			}
private: 	
		devsn* model;
		set imm;
		set new_models;
		set del_models;
		indexed garbage_models;
		pqueue sched;

		void makeStructureChanges (stime_t t);
		void schedule (simulator* sim);
		void route (devs* src_model, const port_value& pv);
		void route (simulator* src, port_t outport);

		SimulatorPool sim_pool; 

		void scheduleComposite ();
		simulator* removeModelFromSchedule (devs* d);
};
