/***************
adevs, a library supporting discrete event simulation
Copyright (C) 2001 James Nutaro

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Bugs, comments, and questions can be sent to nutaro@ece.arizona.edu
***************/
/**
The simulator pool maintains a pool of simulator objects for
use by a coordinator.  Two distinct pools are maintained.  A
pool of sequential (non-threaded) simulators is maintained independently
by each SimulatorPool.  A pool of parallel (threaded) simulators
is also maintained, with these simulators being assigned to models
marked for threading.
*/

class SimulatorPool {

public:		
/// Create an empty simulator pool
		SimulatorPool () {}
/**
Attach a simulator to the given model.  Simulator should be
returned to the pool by calling unwrap.
*/
		simulator* wrap (devs* d);
/// Return a simulator to the pool
		void unwrap (simulator *s);
/// Find the simulator attached to a given model, returns NULL if no simulator is found
		inline simulator* find (const devs& model) {
			return (simulator*)dsmap.find (model);
			}
/// Delete the pool and all simulators provided by it
		~SimulatorPool ();

private:
// Map of models to their simulators
		hashtable dsmap;
// List of sequential simulators 
		indexed sims;
// List of sequential coordinators
		indexed coords;
// List of parallel simulators
		indexed psims;
// List of parallel coordinators
		indexed pcoords;
// Gets a DEVSsimulator from the simulator pool
		simulator* getSequentialSimulator ();
// Gets a DEVScoordinator form the simulator pool
		simulator* getSequentialCoordinator ();
// Gets a parallel DEVSsimulator from the simulator pool
		simulator* getParallelSimulator ();
// Gets a parallel DEVScoordinator form the simulator pool
		simulator* getParallelCoordinator ();

};
