/***************
adevs, a library supporting discrete event simulation
Copyright (C) 2001 James Nutaro

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Bugs, comments, and questions can be sent to nutaro@ece.arizona.edu
***************/
/**
Base class for parallel coordinators and simulators.  Used internally.
This class adds a global thread pool to the simulator class.  These
threads are used to compute I/O and transition functions in parallel.
*/
class psimulator {

public:	
		psimulator(){}
/// Set the process thread limit
		static void setThreadLimit (int limit) { threadLimit = limit; }

protected:
/// Get a thread from the thread pool.  Returns NULL if no threads are available
		Thread* getThread();
/// Return a thread to the thread pool
		void returnThread(Thread* thr);
 
private:	
/// List of available threads
		static indexed thrPool;
/// Mutex to control access to the thread pool
		static Mutex mtx;
/// Process wide thread limit
		static int threadLimit;
/// Number of threads created for the pool
		static int numThreads;

};
