/***************
adevs, a library supporting discrete event simulation
Copyright (C) 1999, 2000, 2001 James Nutaro

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Bugs, comments, and questions can be sent to nutaro@ece.arizona.edu
***************/
/**
A directed graph.
*/
class digraph: public object {
	public:
		digraph();
		digraph(unsigned capacity, unsigned grow = DEFAULT_CONTAINER_GROWTH);
		/// Returns the number of vertices in the graph	
		inline unsigned verticeCount() const { return vertice_count; }
		/// Returns the number of edges in the graph
		inline unsigned edgeCount() const { return edge_count; }
		/// True if (u, v) is an edge, false otherwise
		bool adjacent(const object &u, const object &v) const;
		/// True if u is a vertice, false otherwise
		bool hasVertice(const object &u) const;
		/**
		Returns an indexed list of u's neighbors.  This list is still owned by the 
		digraph, so it should no be tampered with.
		The result is undefined if u is not a vertice in the graph.
		*/
		const indexed* neighbors(const object &u) const;
		/// Adds the object u to the graph	
		void addVertice(object *u);
		/**
		Adds and edge (u, v) using u, v stored in the vertice list.  
		Result is undefined if u, v are not vertices.
		*/
		void addEdge(const object &u, const object &v);
		/**
		Removes u's representation in the graph.  Undefined if u is not a vertice.
		Returns u's representation in the digraph.
		*/
		object * removeVertice(const object &u);
		/// Removes the edge (u, v).  Undefined if (u, v) is not an edge.
		void removeEdge(const object &u, const object &v);
		/// Deletes structure only.  The nodes are preserved.
		~digraph();
	private:	
		unsigned int capacity;
		unsigned int grow;
		hashtable *lists;
		unsigned vertice_count;
		unsigned edge_count;
};
