/***************
adevs, a library supporting discrete event simulation
Copyright (C) 1999, 2000, 2001 James Nutaro

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Bugs, comments, and questions can be sent to nutaro@ece.arizona.edu
***************/
/**
A hashtable.  Key equivalence is defined by the = (const object &) operator.
Hash codes are obtained by calling key.hashcode ()
*/
class hashtable: public object {
	public:
		hashtable();
		/**
		Load/bucket values must be greater than 0.  Load must be less than 1.0.
		Invalid arguments will have undefined effects.
		*/
		hashtable(unsigned int buckets, float loadlimit = DEFAULT_HASH_LOAD);
		/// Copies the structure only
		hashtable(const hashtable &src);
		/**
		Add item, returns the previous instance of an item with that key,
		or NULL if no such item exists. NULL pointers can not be keys or
		values.
		*/
		object* insert(object *key, object *value);
		/**
		Remove returns the saved instance of the key, or NULL if the
		key is not found.
		*/
		object* remove(const object &key);
		/**
		As remove, but the value is deleted if found.
		*/
		object* erase(const object& key);
		/// Returns NULL if key not found.
		object* find(const object &key) const;
		inline unsigned int keyCount() const { return keys; }
		/**
		Returns an arraoy of key objects.  Keys are owned by the hashtable
		and should not be deleted.  The array is owned by the caller and can be 
		removed with a call to delete [].
		*/
		object** getKeys() const;
		/**
		Deletes all buckets, sets the table key count to 0.  Has no effect on
		key/value objects.
		*/
		void clear();
		/// Deletes the hashtable structure, keys and values remain intact
		~hashtable();
	private:	
		unsigned int buckets;
		float loadlimit;
		indexed** table;
		unsigned keys;

		void rehash();

		// Definition of a hash bucket
		class _bucket: public object {
			public:	
				object *key;
				object *value;
				inline _bucket (object *key, object *value):object () { 
					this->key = key; 
					this->value = value; 
				}  
				inline _bucket (const _bucket &src):object (src) {
					this->key = src.key;
					this->value = src.value;
				}
		};

		indexed bucket_pool;
		_bucket* makeBucket(object* key, object* value);
		inline void freeBucket(_bucket* b) { bucket_pool.add(b); }
};
