/***************
adevs, a library supporting discrete event simulation
Copyright (C) 1999, 2000 James Nutaro

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Bugs, comments, and questions can be sent to nutaro@ece.arizona.edu
***************/
/**
A double type wrapped in an object.
*/

class dbl: public object {

public:	double value;

/// Creates a dbl with a value of 0.
inline dbl ():object () { value = 0; }
/// Copy constructor.
inline dbl (const dbl &src):object (src) { value = src.value; }
/// Creates a dbl with the indicated initial value.
inline dbl (double d):object () { value = d; }

inline dbl operator+ (const dbl &b) const { return dbl (value + b.value); }
inline dbl operator+ (const double b) const { return dbl (value + b); }

inline dbl operator- (const dbl &b) const { return dbl (value - b.value); }
inline dbl operator- (const double b) const { return dbl (value - b); }

inline dbl operator* (const dbl &b) const { return dbl (value * b.value); }
inline dbl operator* (const double b) const { return dbl (value * b); }

inline dbl operator/ (const dbl &b) const { return dbl (value / b.value); }
inline dbl operator/ (const double b) const { return dbl (value / b); }

inline bool operator== (const dbl &b) const { return value == b.value; }
inline bool operator== (const double b) const { return value == b; }
inline bool operator!= (const dbl &b) const { return value != b.value; }
inline bool operator!= (const double b) const { return value != b; }

inline bool operator>= (const double b) const { return value >= b; }
inline bool operator>= (const dbl& b) const { return value >= b.value; }

inline bool operator> (const double b) const { return value > b; }
inline bool operator> (const dbl& b) const { return value > b.value; }

inline bool operator<= (const double b) const { return value <= b; }
inline bool operator<= (const dbl& b) const { return value <= b.value; }

inline bool operator< (const double b) const { return value < b; }
inline bool operator< (const dbl& b) const { return value < b.value; }

inline const dbl& operator= (const dbl &b) { value = b.value; return *this; }
inline const dbl& operator= (const double b) { value = b; return *this; }

inline const dbl& operator+= (const dbl &b) { value += b.value; return *this; }
inline const dbl& operator+= (const double b) { value += b; return *this; }

inline const dbl& operator-= (const dbl &b) { value -= b.value; return *this; }
inline const dbl& operator-= (const double b) { value -= b; return *this; }

inline const dbl& operator*= (const dbl &b) { value *= b.value; return *this; }
inline const dbl& operator*= (const double b) { value *= b; return *this; }

inline const dbl& operator/= (const dbl &b) { value /= b.value; return *this; }
inline const dbl& operator/= (const double b) { value /= b; return *this; }

inline const dbl& operator++ () { value++; return *this; }
inline const dbl& operator-- () { value--; return *this; }

virtual bool operator== (const object &obj) const;
virtual bool operator!= (const object &obj) const;
virtual unsigned int hashcode () const;

virtual object* clone () const;

~dbl ();

}; // end of class dbl

