/***************
adevs, a library supporting discrete event simulation
Copyright (C) 1999, 2000 James Nutaro

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Bugs, comments, and questions can be sent to nutaro@ece.arizona.edu
***************/
/**
An integer type wrapped in an object.
*/

class intg: public object {

public:	int value;

/// Creates an intg with initial value of 0.
inline intg ():object () { value = 0; }
/// Copy constructor. 
inline intg (const intg &src):object (src) { value = src.value; }
/// Creates an intg with the indicated initial value.
inline intg (int d):object () { value = d; }

inline intg operator+ (const intg &b) const { return intg (value + b.value); }
inline intg operator+ (const int b) const { return intg (value + b); }

inline intg operator- (const intg &b) const { return intg (value - b.value); }
inline intg operator- (const int b) const { return intg (value - b); }

inline intg operator* (const intg &b) const { return intg (value * b.value); }
inline intg operator* (const int b) const { return intg (value * b); }

inline intg operator/ (const intg &b) const { return intg (value / b.value); }
inline intg operator/ (const int b) const { return intg (value / b); }

/// Mod operator
inline intg operator% (const intg& b) const { return intg (value % b.value); }
/// Mod operator
inline intg operator% (const int b) const { return intg (value % b); }

inline bool operator== (const intg &b) const { return value == b.value; }
inline bool operator== (const int b) const { return value == b; }
inline bool operator!= (const intg &b) const { return value != b.value; }
inline bool operator!= (const int b) const { return value != b; }

inline bool operator>= (const int b) const { return value >= b; }
inline bool operator>= (const intg& b) const { return value >= b.value; }

inline bool operator> (const int b) const { return value > b; }
inline bool operator> (const intg& b) const { return value > b.value; }

inline bool operator<= (const int b) const { return value <= b; }
inline bool operator<= (const intg& b) const { return value <= b.value; }

inline bool operator< (const int b) const { return value < b; }
inline bool operator< (const intg& b) const { return value < b.value; }

inline const intg& operator= (const intg &b) { value = b.value; return *this; }
inline const intg& operator= (const int b) { value = b; return *this; }

inline const intg& operator+= (const intg &b) { value += b.value; return *this; }
inline const intg& operator+= (const int b) { value += b; return *this; }

inline const intg& operator-= (const intg &b) { value -= b.value; return *this; }
inline const intg& operator-= (const int b) { value -= b; return *this; }

inline const intg& operator*= (const intg &b) { value *= b.value; return *this; }
inline const intg& operator*= (const int b) { value *= b; return *this; }

inline const intg& operator/= (const intg &b) { value /= b.value; return *this; }
inline const intg& operator/= (const int b) { value /= b; return *this; }

inline const intg& operator%= (const intg &b) { value %= b.value; return *this; }
inline const intg& operator%= (const int b) { value %= b; return *this; }

inline const intg& operator++ () { value++; return *this; }
inline const intg& operator-- () { value--; return *this; }

virtual bool operator== (const object &obj) const;
virtual bool operator!= (const object &obj) const;
virtual unsigned int hashcode () const;

virtual object* clone () const;

~intg ();

}; // end of class intg

