/***************
adevs, a library supporting discrete event simulation
Copyright (C) 2001 James Nutaro

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Bugs, comments, and questions can be sent to nutaro@ece.arizona.edu
***************/
#include <adevs/adevs.h>
#include <math.h>
using namespace adevs;

// Determines the sign of the value y
double qintg::sign (double y) {

if (y > 0.0) return 1.0;
else if (y < 0.0) return -1.0;
else return 0.0;

}

void qintg::setParams(double D, double e) {

this->D = D;
if (e < 0.0)
	hyst = D / 2.0;
else
	hyst = e;

}

// Initialize the integrator
void qintg::init (double q0, double dq0) {

// set the initial derivative value
x = dq0;
// set the initial state
q = q0;
// determine the quantum level
n = floor (q / D);
// set the time of next event
compute_ta ();

}

// Evaluate the integral over dt units of time
void qintg::integ (double dt, double dq) {

// transition function
q += dt * x;
if (dt >= sigma) n += sign(x);
x = dq;
// compute time advance
compute_ta();

}

void qintg::compute_ta () { 

// compute time advance
if (x > 0.0) 
   // time to reach bottom of block above us
   sigma = fabs (((n+1.0)*D - q) / x);
else if (x < 0.0) 
   // time to reach top of block below us
   sigma = fabs ((q - (n*D-hyst)) / x);
else 
   sigma = INFINITY;
if (sigma < EPSILON) sigma = 0.0;

}
