#include <adevs/adevs.h>
#include <iostream>
#include <algorithm>
#include <assert.h>
#include <math.h>
using namespace adevs;

/**
Some simple quadrature tests for the integrator
*/

void test1 () {

double t = 0.0;
qintg q;
q.setParams(0.1);
q.init(0.0,t);
while (t < 10.0) {
	double dt = min (q.ta(),0.001);
	t += dt;
	q.integ(dt,t);
	}
cout << q.state() << ", expected " << 50.0 << endl;
assert (fabs(q.state() - 50.0) <= 0.001);

}

void test2 () {

double t = -10.0;
qintg q;
q.setParams(0.001);
q.init(0.0,t*t);
while (t < 10.0) {
	double dt = min (q.ta(),0.001);
	t += dt;
	q.integ(dt,t*t);
	}
double expected = pow(10.0,3.0)/3.0 - pow(-10.0,3.0)/3.0;
cout << q.state() << ", expected " << expected << endl;
assert (fabs(q.state() - expected) <= 0.001);

}

void test3 () {

double q0 [2];
double dq [2];
q0[0] = 100.0; q0[1] = 0.0;
dq[0] = 0.0; dq[1] = -9.8;
qode q(2);
q.setParams(0.01);
q.init(q0,dq);
double t = 0.0;
while (q.state(0) > 0.0) {
	t += q.ta();
	dq[0] = q.out(1);
	dq[1] = -9.8; 
	q.integ(q.ta(),dq);
	}
cout << 0.5 * 9.8 * t * t << endl;
double texpected = sqrt(2.0*100.0/9.8);
cout << "Time to ground was " << t << ", expected " << texpected << endl;
assert (fabs(t - texpected) <= 0.01);

}

int main () {

test3();
test1();
test2();

}
