/***************
adevs, a library supporting discrete event simulation
Copyright (C) 1999, 2000, 2001 James Nutaro

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Bugs, comments, and questions can be sent to nutaro@ece.arizona.edu
***************/
#include <adevs/adevs.h>
using namespace adevs;

DScoupled::DScoupled (networkExec* netexec):devsn () { 

this->netexec = netexec;
netexec->set_network (this);

}

DScoupled::DScoupled (const char* name, networkExec* netexec):devsn (name) { 

this->netexec = netexec;
netexec->set_network (this);

}

void DScoupled::_reset (stime_t t0) {

// Skip the full component initialization performed
// by the devsn::_reset method.  The network executive
// will do this for us.
devs::_reset (t0);
// Reset the network executive.  This resets all components
// as well.
netexec->_reset (t0);
// Compute the initial time of next event.
const set* c = netexec->getComponents ();
for (unsigned i = 0; i < c->getSize(); i++) {
   devs* d = (devs*)c->getObj(i); 
   if (d->timeNext() < timeNext())
      hold (d->sigma());
   }

}

const indexed* DScoupled::computeNeighbors (devs& model, const port_value& pv) {

return netexec->computeNeighbors (model, pv);

}

void DScoupled::makeStructureChanges (set& add_models, set& del_models) {

netexec->makeStructureChanges (add_models, del_models);

}

const set* DScoupled::getComponents () const {

return netexec->getComponents ();

}

DScoupled::~DScoupled () { 

delete netexec;

}
