/***************
adevs, a library supporting discrete event simulation
Copyright (C) 1999, 2000, 2001 James Nutaro

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Bugs, comments, and questions can be sent to nutaro@ece.arizona.edu
***************/
#include <adevs/adevs.h>
using namespace adevs;

networkExec::networkExec ():atomic () { 

add (this);
network = NULL;

} 

networkExec::networkExec (const char* name):atomic (name) {

add (this);
network = NULL;

}

void networkExec::_reset (stime_t t) {

// Calls the networkExec's user defined reset() method
atomic::_reset (t);
// Remove new models from new list (they've already been added)
new_models.removeAll ();
// Clear the deleted model list 
del_models.empty ();
// Reset the component models (except this one)
for (unsigned i = 0; i < components.getSize(); i++) {
   devs* model = (devs*)components[i];
   if (model != this) model->_reset (t);
   }

}

void networkExec::add (devs* model) {

new_models.add (model);
components.add (model);

}

void networkExec::remove (devs* model) {

// Don't allow the executive to be removed
if (model == this) return;

// Otherwise, if the component exists, remove it
if (components.instanceCount (*model) > 0) {
   del_models.add (model);
   components.remove (components.indexOf (*model));
   }
}

const set* networkExec::getComponents () const {

return &components;

}

void networkExec::makeStructureChanges (set& _new_models, set& _del_models) {

_new_models.assignUnion (new_models);
_del_models.assignUnion (del_models);
new_models.removeAll ();
del_models.removeAll ();

}

networkExec::~networkExec () { 

components.remove (components.indexOf (*this));
new_models.empty ();
del_models.empty ();
components.empty ();

}
