/***************
adevs, a library supporting discrete event simulation
Copyright (C) 2001 James Nutaro

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Bugs, comments, and questions can be sent to nutaro@ece.arizona.edu
***************/
#include <adevs/adevs.h>
using namespace adevs;

void adevs::coord_compute_next_output (void* arg) {

pDEVScoordinator* coord = (pDEVScoordinator*)arg;
coord->_computeNextOutput (coord->t_thread);
coord->mtx.unlock ();

}

void adevs::coord_compute_io (void* arg) {

pDEVScoordinator* coord = (pDEVScoordinator*)arg;
coord->_computeIO (coord->t_thread);
coord->mtx.unlock ();

}

void adevs::coord_compute_transition (void* arg) {

pDEVScoordinator* coord = (pDEVScoordinator*)arg;
coord->_computeTransition (coord->t_thread);
coord->mtx.unlock ();

}

pDEVScoordinator::pDEVScoordinator ():
DEVScoordinator (simulator::PCOORD) { 

t_thread = 0.0;
thr = NULL;

}

void pDEVScoordinator::computeNextOutput (stime_t t) {

if (thr == NULL)
	thr = getThread();
if (thr == NULL)
	DEVScoordinator::computeNextOutput(t);
else {
	mtx.lock ();
	t_thread = t;
	thr->run (coord_compute_next_output, this);
	}

}

void pDEVScoordinator::_computeNextOutput (stime_t t) {

DEVScoordinator::computeNextOutput (t);

}

void pDEVScoordinator::computeIO (stime_t t) {

if (thr == NULL)
	thr = getThread();
if (thr == NULL)
	DEVScoordinator::computeIO(t);
else {
	mtx.lock ();
	t_thread = t;
	thr->run (coord_compute_io, this);
	}

}

void pDEVScoordinator::_computeIO (stime_t t) {

DEVScoordinator::computeIO (t);

}

void pDEVScoordinator::computeTransition (stime_t t) {

if (thr == NULL)
	thr = getThread();
if (thr == NULL)
	DEVScoordinator::computeTransition(t);
else {
	mtx.lock ();
	t_thread = t;
	thr->run (coord_compute_transition, this);
	}

}


void pDEVScoordinator::_computeTransition (stime_t t) {

DEVScoordinator::computeTransition (t);

}

void pDEVScoordinator::wait () {

if (thr != NULL) {
	mtx.lock ();
	mtx.unlock ();
	returnThread(thr);
	thr = NULL;
	}

}

pDEVScoordinator::~pDEVScoordinator () { }
