/***************
adevs, a library supporting discrete event simulation
Copyright (C) 1999, 2000 James Nutaro

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Bugs, comments, and questions can be sent to nutaro@ece.arizona.edu
***************/
#include <adevs/adevs.h>
using namespace adevs;

staticDigraph::staticDigraph ():coupled () { 

net = new digraph ();

}

staticDigraph::staticDigraph (const char* name):coupled (name) { 

net = new digraph ();

}

void staticDigraph::couple (devs* src, port_t srcPort,
				devs* dst, port_t dstPort) {

const set* models = getComponents ();

if (models->instanceCount (*src) == 0 && src != this) {
   UndefinedException err (src);
   throw err;
   } 

else if (models->instanceCount (*dst) == 0 && dst != this) {
   UndefinedException err (dst);
   throw err;
   } 

EventReceiver ersrc (src, srcPort);
EventReceiver erdst (dst, dstPort);
EventReceiver* er;

if (!net->hasVertice (ersrc)) {
   er = new EventReceiver (ersrc);
   net->addVertice (er);
   nodes.add (er);
   }
 
if (!net->hasVertice (erdst)) {
   er = new EventReceiver (erdst);
   net->addVertice (er);
   nodes.add (er);
   }

if (!net->adjacent (ersrc, erdst))
   net->addEdge (ersrc, erdst);

}

const indexed* staticDigraph::computeNeighbors (devs& src, 
const port_value& pv) {

EventReceiver er (&src, pv.port);
return net->neighbors (er);

}

staticDigraph::~staticDigraph () {

delete net;
nodes.empty ();

}
