#include <adevs/adevs.h>
#include "systolicArray.h"
#include "systolicCell.h"
using namespace adevs;

// define the ports
const port_t systolicArray::xin = 0;
const port_t systolicArray::xout = 1;
const port_t systolicArray::yin = 2;
const port_t systolicArray::yout = 3;

/**
Constructor.  Builds the array and assigns weights to the cells.
*/
systolicArray::systolicArray (const double* weights, unsigned length):
staticDigraph () {

systolicCell* s1;
systolicCell* s2;

s1 = new systolicCell (weights[length - 1]);

// add the ports
addInputPort (xin);
addInputPort (yin);
addOutputPort (xout);
addOutputPort (yout);

// add the first cell and establish its external coupling.
add (s1);
couple (this, xin, s1, s1->xin);
couple (this, yin, s1, s1->yin);

s2 = s1;

// iterate through the array; set weights and establish internal
// (inter-cell) coupling.
for (unsigned i = 1; i < length; i++) {

   s1 = new systolicCell (weights[length - 1 - i]);
   add (s1);
   add (s2);
   couple (s2, s2->xout, s1, s1->xin);
   couple (s2, s2->yout, s1, s1->yin);
   s2 = s1;

   }

// external output coupling
couple (s2, s2->xout, this, xout);
couple (s2, s2->yout, this, yout);

}

systolicArray::~systolicArray () { }
