#include <adevs/adevs.h>
#include <stdlib.h>
#include <iostream>
#include <assert.h>
#include "systolicCell.h"
using namespace adevs;

// assign values to the ports.
const port_t systolicCell::yin = 0;
const port_t systolicCell::yout = 1;
const port_t systolicCell::xin = 2;
const port_t systolicCell::xout = 3;

/**
Construct the sytolicCell.  Adds input and output ports and
sets up the cells weighting factor.
*/
systolicCell::systolicCell (double weight):atomic () {

x = x_in = y_out = NULL;
w = weight;

addInputPort (yin);
addInputPort (xin);
addOutputPort (yout);
addOutputPort (xout);

}

/**
Reset/initialization function.
*/
void systolicCell::reset () {

// clear the cell's memory
x_in = x = y_out = NULL;
// wait until a value comes in.
passivate ();

}

/**
Internal transition function.
*/
void systolicCell::delta_int () { 

// An output without an input indicates that this cell
// has no more computation to perform, so passivate.
if (x_in == NULL) passivate ();
// otherwise, generate an output at the start of the next cycle.
else hold (1);
// assign x_in to x, clear y_out and x_in
x = x_in;
y_out = x_in = NULL;
 
}

/**
External transition function
*/
void systolicCell::delta_ext () {

// get the x and y inputs
bag* _y = inputOnPort (yin);
bag* _x = inputOnPort (xin);

for (unsigned i = 0; i < _x->getSize(); i++) 
   x_in = (dbl*)_x->getObj(i)->clone ();

// compute the next y output
for (unsigned i = 0; i < _y->getSize(); i++) {
   assert (x_in != NULL);
   y_out = (dbl*)_y->getObj(i)->clone (); 
   y_out->value = y_out->value + w * x_in->value;
   }

// hold until the next cycle
hold (1);

}

/**
Confluent transition function.
*/
void systolicCell::delta_conf () {

// internal the external.  This results in some unneccesary work
// in terms of rescheduling the model twice during a transition.
delta_int ();
delta_ext ();

}

/**
Output function.
*/
void systolicCell::output_func () {

// if we have an x value, output it
if (x != NULL) 
   output (xout, x);
// if we have a new y value, output it
if (y_out != NULL) 
   output (yout, y_out);

}

systolicCell::~systolicCell () {

if (x != NULL) delete x;
if (x_in != NULL) delete x_in;
if (y_out != NULL) delete y_out;

} 
