/***********
Copyright (C) 1999, 2000  James Nutaro

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Bugs, comments, and questions can be sent to nutaro@ece.arizona.edu
***************/
/**

Events that are generated/received by the DEVS models.

*/


// Indicates a collision between the target object and the indicated shape.
class collisionEvent: public adevs::object {

public:
	inline collisionEvent (const adevs::object* target, const shape* geom):
		object() { 
		this->target = target; this->geom = geom;
		}
	inline const adevs::object* getTarget () const { return target; }
	inline const shape* getGeometry () const { return geom; }
	
private:
	const adevs::object* target;
	const shape* geom;

};

// Indicates significant motion by the source object.
class motionEvent: public adevs::object {

public:
	inline motionEvent (const adevs::object* source, const vector& velocity,
		const shape* geom):adevs::object () {
		this->source = source;
		this->velocity = velocity;
		this->geom = geom;
		}
	inline motionEvent (const adevs::object* source, const shape* geom):
		adevs::object () {
		this->source = source;
		this->geom = geom;
		}
	inline const adevs::object* getSource () const { return source; }
	inline const shape* getPosition () const { return geom; }
	inline const vector& getVelocity () const { return velocity; }

private:
	const adevs::object* source;
	const shape* geom;
	vector velocity;

};

// Used to report the status of the source object.
enum eStatus { OK, DESTROYED, INVULNERABLE };

class statusEvent: public adevs::object {

public:
	inline statusEvent (const adevs::object* source, eStatus status, 
				const shape* geom):
		adevs::object () {
		this->source = source;
		this->status = status;
		this->geom = geom;
		}
	inline eStatus getStatus () const { return status; }
	inline const adevs::object* getSource () const { return source; }
	inline const shape* getGeometry () const { return geom; }

private:
	eStatus status;
	const adevs::object* source;
	const shape* geom;

};

// A paddle control event (generated by the player).
enum ePaddleControl { LEFT, RIGHT, STOP };

class paddleControlEvent: public adevs::object {

public:
	inline paddleControlEvent (ePaddleControl sig):
		adevs::object () {
		this->sig = sig;
		}
	inline ePaddleControl getControl () const { return sig; }

private:
	ePaddleControl sig;

};
