#include <adevs/adevs.h>
#include <stdlib.h>
#include "delay.h"
using namespace adevs;

const port_t delay::in = 0;
const port_t delay::out = 1;

delay::delay (stime_t t):atomic () {

dt = t;

addInputPort (in);
addOutputPort (out);

}

void delay::reset () {

object* o;

while (items.getSize () > 0) {
   o = items.removeMinimum ();
   delete o;
   }

for (unsigned i = 0; i < next.getSize (); i++) {
   o = next[i];
   delete o;
   }

next.removeAll ();

passivate ();

}

void delay::schedule () {

if (items.getSize () == 0) passivate ();

else {

   stime_t tn = items.minPriority ();
   stime_t t = tn;

   while (items.getSize () > 0 && t == tn) {
  
      t = items.minPriority ();
      if (t == tn) next.add (items.removeMinimum ());
      
      }

   hold (dt - (tn - timeCurrent ()));

   }

}

void delay::delta_int () {

for (unsigned i = 0; i < next.getSize (); i++) {
   delete next[i];
   }

next.removeAll ();

schedule ();

}

void delay::delta_ext () {

bag* input = inputOnPort (in);

for (unsigned i = 0; i < input->getSize (); i++) {
   object* obj = input->getObj (i);
   items.insert (obj->clone (), timeCurrent ());
   }

if (timeCurrent () == timeLast () || timeNext () == INFINITY)
   schedule ();
 
}

void delay::delta_conf () {

delta_int ();

bag* input = inputOnPort (in);

for (unsigned i = 0; i < input->getSize (); i++) {
   object* obj = input->getObj (i);
   items.insert (obj->clone (), timeCurrent ());
   }

}

void delay::output_func () {

for (unsigned i = 0; i < next.getSize (); i++) {
   object* o = next[i];
   output (out, o->clone ());
   }

}

delay::~delay () { 

}
