#include <adevs/adevs.h>
#include <stdlib.h>
#include "genr.h"
#include <iostream>
using namespace adevs;

const port_t genr::start = 0;
const port_t genr::stop = 1;
const port_t genr::signal = 2;

genr::genr (const stime_t* pattern, unsigned length, 
		unsigned iterations, bool active):
atomic () {

this->active = init_state = active;
this->length = length;
count = 0;
this->iterations = iterations;

this->pattern = (stime_t*)malloc (sizeof (stime_t) * length);

for (unsigned i = 0; i < length; i++) 
   this->pattern[i] = pattern[i];

addInputPort (start);
addInputPort (stop);
addOutputPort (signal);

}

genr::genr (stime_t period, unsigned iterations, bool active):
atomic () {

this->active = init_state = active;
this->length = 1;
count = 0;
this->iterations = iterations;

this->pattern = (stime_t*)malloc (sizeof (stime_t));
*this->pattern = period;

addInputPort (start);
addInputPort (stop);
addOutputPort (signal);

}

void genr::reset () {

active = init_state;
count = 0;

if (!active) passivate ();
else hold (pattern[count++%length]);

}

void genr::delta_int () { 

if (count >= iterations) passivate ();
else hold (pattern[count++%length]); 

}

void genr::delta_ext () {

passivate ();
active = false;

bag* value = inputOnPort (start);

if (value->getSize () > 0) {
   active = true;
   count = 0;
   hold (pattern[count++]);
   }

else { 
   std::cout << "Got genr.stop" << std::endl;
   }

}

void genr::delta_conf () { delta_ext (); }

void genr::output_func () { output (signal, new intg (count)); }

genr::~genr () {

free (pattern);

} 

